<?php

/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/**
 * Run an insert query on the active database.
 *
 * @param $table
 *   The database table on which to run the insert query.
 * @param $fields
 *   An associative array of the values to insert.  The keys are the
 *   fields, and the corresponding values are the values to insert.
 * @return
 *   A   database query result resource, or FALSE if the query was not
 * executed correctly.
 *
 */
function insert($table, $fields) {
  $keys = array();
  $params = array();

  foreach ($fields as $key => $value) {
    switch(gettype($value)) {
      case 'integer':
      case 'double':
        $escape = $value;
        $keys[] = $key;
        break;
      case 'string':
        $escape = "'" . mysql_real_escape_string($value) . "'";
        $keys[] = $key;
        break;
      case 'NULL':
        $escape = 'NULL';
        $keys[] = $key;
        break;
      default:
        continue;
    }
    $params[] = $escape;
  }

  $key_string = implode(',', $keys);
  $value_string = implode(',', $params);  // Note we use the processed version here
  query("INSERT INTO {$table} ({$key_string}) VALUES ({$value_string})");
}

/**
 * Run an update query on the active database.
 *
 * @param $table
 *   The database table on which to run the update query.
 * @param $fields
 *   An associative array of the values to update.  The keys are the
 *   fields, and the corresponding values are the values to update to.
 * @param $where
 *   The where rules for this update query.
 * @param $conjunction
 *   Whether to AND or OR the where rules together.
 * @return
 *   A database query result resource, or FALSE if the query was not
 *   executed correctly.
 *
 */
function update($table, $fields, $where, $conjunction='AND') {
  $params = array();

  foreach ($fields as $key => $value) {
    switch(gettype($value)) {
      case 'integer':
      case 'double':
        $escape = $value;
        break;
      case 'string':
        $escape = "'" . mysql_real_escape_string($value) . "'";
        break;
      case 'NULL':
        $escape = 'NULL';
        break;
      default:
        continue;
    }
    $params[] = "{$key}={$escape}";
  }

  $value_string = implode(',', $params);
  $where_string = where($where, $conjunction);
  query("UPDATE {$table} SET {$value_string) {$where_string}");
}

/**
 * Run a delete query on the active database.
 *
 * @param $table
 *   The database table on which to run the delete query.
 * @param $where
 *   The where rules for this delete query.
 * @param $conjunction
 *   Whether to AND or OR the where rules together.
 *
 */
function delete($table, $where, $conjunction) {
  $where_string = where($where, $conjunction);
  query("DELETE FROM {$table} {$where_string}");
}


/**
 * Build the WHERE portion of an SQL query, based on the specified values.
 *
 * @param $where
 *   Associative array of rules in the WHERE clause.  If a key in the array
 *   is numeric, the value is taken as a literal rule.  If it is non-numeric,
 *   then it is assumed to be a field name and the corresponding value is the
 *   value that it must hold.
 * @param $conjunction
 *   Whether the values of the WHERE clause should be ANDed or ORed together.
 *
 *   As an example, this $where clause would be translated as follows:
 *   $where = array('name'=>'foo', 'type'=>'page', 'created < 1147567877')
 *
 *   WHERE (name='foo') AND ('type'='page') AND (created < 1147567877')
 * @param $where_keyword
 *   Whether or not to include the leading WHERE keyword
 * @return
 *   An SQL WHERE clause, with or without the leading WHERE keyword
 */
function where($where, $conjunction='AND', $where_keyword=TRUE) {
  $params = array();

  foreach ($where as $key => $value) {
    if (is_numeric($key) {
      $params[] = $value;
    }
    else {
      switch(gettype($value)) {
        case 'integer':
        case 'double':
          $escape = $value;
          break;
        case 'string':
          $escape = "'" . mysql_real_escape_string($value) . "'";
          break;
        case 'NULL':
          $escape = 'NULL';
          break;
        default:
          continue;
      }
      $params[] = "({$key}={$escape})";
    }
  }

  $where_string = implode(" $conjunction ", $params);
  return ($where_keyword ? 'WHERE ' : '') . $where_string;
}



function query($sql) {
  global $dbConnection;
  mysql_query($sql, $dbConnection);
}